eureqa_colors = {
    "WHITE": "#fff",
    "GREEN": "#72bb3a",
    "ORANGE": "#e77219",
    "YELLOW": "#FCD051",
    "RED": "#c91f3e",
    "GOLD": "#edb619",
    "PRIMARY_BLUE": "#00a4c9",
    "SECONDARY_BLUE": "#6296C3",
    "LINK_BLUE": "#006699",
    "LIGHT_GREY": "#ddd",
    "MEDIUM_GREY": "#aaa",
    "DARK_GREY": "#5f605f",
    "ICON_GREY": "#5f605f",
    "DEFAULT_FONT": "#333333",
    "LIGHT_FONT": "#aaa",
    "BACKGROUND_GREY": "#f4f4f4",
    "PRIMARY_BUTTON": "#e77219",
    "SECONDARY_BUTTON": "#777",
    "BOX_SHADOW": "0px 2px 3px 0px #ddd",
    "BORDER_LIGHT": "1px solid #ddd",
    "BORDER_MEDIUM": "1px solid #aaa",
    "BORDER_DOTTED": "1px dotted #aaa",
    "BORDER_HIGHLIGHT": "1px solid #00a4c9",
    "TABLE_ROW_SELECTED": "1px double #00a4c9"
}